from __future__ import unicode_literals, absolute_import

import shutil
import tempfile

import boto3
import robosignatory.utils as utils
import robosignatory.work

import logging
log = logging.getLogger("robosignatory.coreosconsumer")


class CoreOSSigner(object):

    def __init__(self, config):
        self.config = config

        aws_config = self.config['coreos']['aws']
        s3 = boto3.resource(
            's3',
            region_name=aws_config['region'],
            aws_access_key_id=aws_config['access_key'],
            aws_secret_access_key=aws_config['access_secret'],
        )
        bucket_name = self.config['coreos']['bucket']
        self.bucket = s3.Bucket(bucket_name)

        signing_config = self.config['signing']
        self.signer = utils.get_signing_helper(**signing_config)

        log.info('CoreOSSigner ready for service')

    def consume(self, msg):
        # Message structure:
        # https://github.com/coreos/fedora-coreos-tracker/issues/198#issuecomment-513944390
        log.info(
            'CoreOS wants to sign '
            '%(build_id)s on %(stream)s for %(basearch)s' % msg.body
        )
        if msg.topic.endswith('.coreos.build.request.artifacts-sign'):
            objects = msg.body["artifacts"]
        elif msg.topic.endswith('.coreos.build.request.ostree-sign'):
            objects = [{
                'file': msg.body["commit_object"],
                'checksum': msg.body["checksum"],
            }]
        self.sign_objects(objects)

    def sign_objects(self, objects):
        # Evaluation of the key is here and not in __init__ because we may want
        # a stream or version-dependant key in the future.
        key = self.config["coreos"]["key"]
        tmpdir = tempfile.mkdtemp(prefix="/tmp/robosignatory-")
        try:
            for obj in objects:
                robosignatory.work.process_coreos(
                    self.signer, key, self.bucket, tmpdir, obj)
        finally:
            shutil.rmtree(tmpdir)
